@@ -6,12 +6,12 @@ extern crate lazy_static;
 pub mod error;
 
 macro_rules! partitions_from_table_source {
-    ($table_source:ident, $call_with_r:expr) => {{
+    ($table_source:ident, $call_with_r:expr, $ctx:ident) => {{
         use crate::io;
         use std::convert::TryFrom;
 
         let uri = $table_source.parsed_uri()?;
-        match io::BlobStoreType::try_from(uri.scheme())? {
+        match io::BlobStoreType::try_from(uri.scheme()).context(table::IoSnafu)? {
             io::BlobStoreType::FileSystem => {
                 io::fs::partitions_from_uri(&$table_source, uri, $call_with_r)
             }
@@ -19,7 +19,8 @@ macro_rules! partitions_from_table_source {
                 io::http::partitions_from_uri(&$table_source, uri, $call_with_r).await
             }
             io::BlobStoreType::S3 | io::BlobStoreType::GCS | io::BlobStoreType::Azure => {
-                io::object_store::partitions_from_uri(&$table_source, uri, $call_with_r).await
+                io::object_store::partitions_from_uri(&$table_source, uri, $call_with_r, &$ctx)
+                    .await
             }
             io::BlobStoreType::Memory => {
                 io::memory::partitions_from_memory(&$table_source, $call_with_r).await
@@ -36,6 +37,7 @@ pub mod table;
 
 pub use crate::columnq::*;
 
+pub use arrow_schema;
 /// export datafusion and arrow so downstream won't need to declare dependencies on these libraries
 pub use datafusion;
 pub use datafusion::arrow;
