@@ -4,6 +4,7 @@ use arrow_flight::flight_service_server::FlightServiceServer;
 use arrow_flight::sql::metadata::{
     SqlInfoData, SqlInfoDataBuilder, XdbcTypeInfo, XdbcTypeInfoData, XdbcTypeInfoDataBuilder,
 };
+use arrow_flight::sql::server::PeekableFlightDataStream;
 use arrow_flight::sql::{
     server::FlightSqlService, ActionBeginSavepointRequest, ActionBeginSavepointResult,
     ActionBeginTransactionRequest, ActionBeginTransactionResult, ActionCancelQueryRequest,
@@ -18,15 +19,17 @@ use arrow_flight::sql::{
     SqlInfo, TicketStatementQuery, XdbcDataType,
 };
 use arrow_flight::{
-    flight_service_server::FlightService, Action, FlightData, FlightDescriptor, FlightEndpoint,
-    FlightInfo, HandshakeRequest, HandshakeResponse, IpcMessage, SchemaAsIpc, Ticket,
+    flight_service_server::FlightService, Action, FlightDescriptor, FlightEndpoint, FlightInfo,
+    HandshakeRequest, HandshakeResponse, IpcMessage, SchemaAsIpc, Ticket,
 };
 use async_trait::async_trait;
+use base64::prelude::*;
 use columnq::arrow_schema::Schema;
 use columnq::datafusion::arrow::ipc::writer::IpcWriteOptions;
 use columnq::datafusion::arrow::record_batch::RecordBatch;
 use columnq::datafusion::logical_expr::LogicalPlan;
 use columnq::datafusion::prelude::{DataFrame, SessionContext};
+use constant_time_eq::constant_time_eq;
 use dashmap::DashMap;
 use futures::{Stream, StreamExt, TryStreamExt};
 use log::{debug, info};
@@ -41,14 +44,10 @@ use tonic::transport::{Certificate, Identity, ServerTlsConfig};
 use tonic::{Request, Response, Status, Streaming};
 use uuid::Uuid;
 
-use crate::config::Config;
+use crate::config::{BasicAuth, Config};
 use crate::context::RoapiContext;
 use crate::server::RunnableServer;
 
-// FIXME: uncomment for arrow 0.47 upgrade
-// use arrow_flight::sql::server::PeekableFlightDataStream;
-type PeekableFlightDataStream = Streaming<FlightData>;
-
 macro_rules! internal_error {
     ($desc:expr, $err:expr) => {
         Status::internal(format!("{}: {} at {}:{}", $desc, $err, file!(), line!()))
@@ -59,8 +58,9 @@ macro_rules! internal_error {
 // see: https://docs.rs/datafusion/latest/datafusion/execution/context/struct.SessionState.html#method.resolve_table_references
 const CATALOG_NAME: &str = "roapi";
 const SCHEMA_NAME: &str = "public";
-const FAKE_TOKEN: &str = "uuid_token";
 const FAKE_UPDATE_RESULT: i64 = 1;
+const AUTH_HEADER: &str = "authorization";
+const BEARER_PREFIX: &str = "Bearer ";
 
 static INSTANCE_SQL_DATA: Lazy<SqlInfoData> = Lazy::new(|| {
     let mut builder = SqlInfoDataBuilder::new();
@@ -108,14 +108,20 @@ pub struct RoapiFlightSqlService<H: RoapiContext> {
     ctx: Arc<H>,
     statements: Arc<DashMap<String, LogicalPlan>>,
     results: Arc<DashMap<String, Vec<RecordBatch>>>,
+    auth_token: Option<String>,
+    auth_basic_encoded: Option<String>,
 }
 
 impl<H: RoapiContext> RoapiFlightSqlService<H> {
-    fn new(ctx: Arc<H>) -> Self {
+    fn new(ctx: Arc<H>, auth_token: Option<String>, auth_basic: Option<BasicAuth>) -> Self {
         Self {
             ctx,
             statements: Arc::new(DashMap::new()),
             results: Arc::new(DashMap::new()),
+            auth_token,
+            auth_basic_encoded: auth_basic.map(|auth| {
+                BASE64_STANDARD_NO_PAD.encode(format!("{}:{}", auth.username, auth.password))
+            }),
         }
     }
 
@@ -125,9 +131,9 @@ impl<H: RoapiContext> RoapiFlightSqlService<H> {
         Ok(self.ctx.get_dfctx().await)
     }
 
-    fn get_result(&self, handle: &str) -> Result<Vec<RecordBatch>, Status> {
-        if let Some(result) = self.results.get(handle) {
-            Ok(result.clone())
+    fn pop_result(&self, handle: &str) -> Result<Vec<RecordBatch>, Status> {
+        if let Some((_, result)) = self.results.remove(handle) {
+            Ok(result)
         } else {
             Err(Status::internal(format!(
                 "Request handle not found: {handle}"
@@ -154,25 +160,26 @@ impl<H: RoapiContext> RoapiFlightSqlService<H> {
     }
 
     fn check_token<T>(&self, req: &Request<T>) -> Result<(), Status> {
-        let metadata = req.metadata();
-        if let Some(auth) = metadata.get("authorization") {
-            let s = auth
+        if let Some(token) = &self.auth_token {
+            let metadata = req.metadata();
+            let auth_header = metadata
+                .get(AUTH_HEADER)
+                .ok_or_else(|| Status::unauthenticated("token not found"))?;
+            let auth_header = auth_header
                 .to_str()
                 .map_err(|e| Status::internal(format!("Error parsing header: {e}")))?;
 
-            let authorization = s.to_string();
-            let bearer = "Bearer ";
-            if !authorization.starts_with(bearer) {
-                Err(Status::internal("Invalid auth header!"))?;
+            if !auth_header.starts_with(BEARER_PREFIX) {
+                Err(Status::internal("invalid auth type"))?;
             }
-            let token = authorization[bearer.len()..].to_string();
-            if token == FAKE_TOKEN {
-                return Ok(());
-            } else {
-                return Err(Status::unauthenticated("invalid token "));
+            if auth_header.len() <= BEARER_PREFIX.len() {
+                return Err(Status::unauthenticated("invalid token"));
+            }
+            let user_token = &auth_header[BEARER_PREFIX.len()..];
+            if !constant_time_eq(token.as_bytes(), user_token.as_bytes()) {
+                return Err(Status::unauthenticated("invalid token"));
             }
         }
-
         Ok(())
     }
 }
@@ -202,18 +209,69 @@ impl<H: RoapiContext> FlightSqlService for RoapiFlightSqlService<H> {
 
     async fn do_handshake(
         &self,
-        _request: Request<Streaming<HandshakeRequest>>,
+        request: Request<Streaming<HandshakeRequest>>,
     ) -> Result<
         Response<Pin<Box<dyn Stream<Item = Result<HandshakeResponse, Status>> + Send>>>,
         Status,
     > {
+        let auth_basic_encoded = self
+            .auth_basic_encoded
+            .as_ref()
+            .ok_or_else(|| Status::unauthenticated("no basic auth cred configured"))?;
+
+        let auth = request
+            .metadata()
+            .get(AUTH_HEADER)
+            .ok_or_else(|| Status::unauthenticated("missing authorization header"))?;
+
+        let (auth_type, auth_value) = auth
+            .to_str()
+            .map_err(|_| Status::internal("Failed to parse authorization header"))?
+            .split_once(' ')
+            .ok_or_else(|| Status::invalid_argument("Invalid authorization header"))?;
+
+        if auth_type.to_lowercase() != "basic" {
+            return Err(Status::invalid_argument(
+                "Invalid authorization type, basic auth is the only supported type",
+            ));
+        }
+
+        // auth_value could contain `==` base64 paddings, while auth_basic_encoded doesn't
+        if auth_basic_encoded.len() > auth_value.len()
+            || !constant_time_eq(
+                auth_basic_encoded.as_bytes(),
+                auth_value[..auth_basic_encoded.len()].as_bytes(),
+            )
+        {
+            return Err(Status::unauthenticated("unauthorized"));
+        }
+
+        let token = self
+            .auth_token
+            .as_ref()
+            .ok_or_else(|| Status::internal("token not found"))?;
+        let auth_header = format!("{}{}", BEARER_PREFIX, &token);
         let result = HandshakeResponse {
             protocol_version: 0,
-            payload: FAKE_TOKEN.into(),
+            payload: token.clone().into(),
         };
         let result = Ok(result);
         let output = futures::stream::iter(vec![result]);
-        return Ok(Response::new(Box::pin(output)));
+
+        let mut resp: Response<
+            Pin<Box<dyn Stream<Item = Result<HandshakeResponse, Status>> + Send>>,
+        > = Response::new(Box::pin(output));
+        // Reference implementation returns token in auth header instead of payload, see:
+        // https://github.com/apache/arrow/blob/6a7a6ee308b69c12f46f874cb3d52892e172d7b7/go/arrow/flight/client.go#L335
+        // https://github.com/apache/arrow/blob/6a7a6ee308b69c12f46f874cb3d52892e172d7b7/cpp/src/arrow/flight/transport/grpc/grpc_client.cc#L449C54-L449C65
+        resp.metadata_mut().insert(
+            AUTH_HEADER,
+            auth_header
+                .parse()
+                .map_err(|_| Status::internal("failed to encode auth header"))?,
+        );
+
+        Ok(resp)
     }
 
     async fn do_get_fallback(
@@ -238,9 +296,9 @@ impl<H: RoapiContext> FlightSqlService for RoapiFlightSqlService<H> {
         let handle = fr.handle;
 
         info!("getting results for {handle}");
-        let result = self.get_result(&handle)?;
+        let result = self.pop_result(&handle)?;
         // if we get an empty result, create an empty schema
-        let (schema, batches) = match result.get(0) {
+        let (schema, batches) = match result.first() {
             None => (Arc::new(Schema::empty()), vec![]),
             Some(batch) => (batch.schema(), result.clone()),
         };
@@ -286,7 +344,7 @@ impl<H: RoapiContext> FlightSqlService for RoapiFlightSqlService<H> {
             .map_err(|e| internal_error!("Error executing query", e))?;
 
         // if we get an empty result, create an empty schema
-        let schema = match result.get(0) {
+        let schema = match result.first() {
             None => Schema::empty(),
             Some(batch) => (*batch.schema()).clone(),
         };
@@ -364,7 +422,7 @@ impl<H: RoapiContext> FlightSqlService for RoapiFlightSqlService<H> {
             .map_err(|e| internal_error!("Error executing query", e))?;
 
         // if we get an empty result, create an empty schema
-        let schema = match result.get(0) {
+        let schema = match result.first() {
             None => Schema::empty(),
             Some(batch) => (*batch.schema()).clone(),
         };
@@ -632,7 +690,7 @@ impl<H: RoapiContext> FlightSqlService for RoapiFlightSqlService<H> {
         // TODO: ignore SYSTEM TABLE and VIEW
         let table_type = query
             .table_types
-            .get(0)
+            .first()
             .map(|s| s.as_str())
             .unwrap_or_else(|| "table")
             .to_string();
@@ -908,28 +966,8 @@ pub struct RoapiFlightSqlServer<H: RoapiContext> {
     pub ctx: Arc<H>,
     pub addr: std::net::SocketAddr,
     pub tls_config: Option<ServerTlsConfig>,
-}
-
-fn tonic_server_builder<H: RoapiContext>(
-    ctx: Arc<H>,
-    tls_config: &Option<ServerTlsConfig>,
-) -> Result<tonic::transport::server::Router<tower_layer::Identity>, Whatever> {
-    let svc = FlightServiceServer::with_interceptor(
-        RoapiFlightSqlService::new(ctx),
-        |req: Request<()>| -> Result<Request<()>, Status> {
-            info!("FlightSQL req: {:?}", &req);
-            Ok(req)
-        },
-    );
-    let mut builder = tonic::transport::Server::builder();
-    if let Some(cfg) = tls_config {
-        builder = whatever!(
-            builder.tls_config(cfg.clone()),
-            "Failed to build TLS config"
-        );
-    }
-
-    Ok(builder.add_service(svc))
+    pub auth_token: Option<String>,
+    pub auth_basic: Option<BasicAuth>,
 }
 
 impl<H: RoapiContext> RoapiFlightSqlServer<H> {
@@ -962,12 +1000,40 @@ impl<H: RoapiContext> RoapiFlightSqlServer<H> {
             })
             .transpose()?;
 
+        let auth_basic = config
+            .flight_sql_config
+            .as_ref()
+            .and_then(|c| c.auth_basic.as_ref())
+            .cloned();
+        let auth_token = match (
+            config
+                .flight_sql_config
+                .as_ref()
+                .and_then(|c| c.auth_token.as_ref())
+                .cloned(),
+            &auth_basic,
+        ) {
+            (Some(token), None) => Some(token),
+            // when both basic auth and token auth are specified, handshake will return the
+            // specified token
+            (Some(token), Some(_)) => Some(token),
+            // when only basic auth is specified, handshake will return encoded basic auth
+            // value as token to keep it constant
+            (None, Some(BasicAuth { username, password })) => {
+                let token = BASE64_STANDARD_NO_PAD.encode(format!("{}:{}", username, password));
+                Some(token)
+            }
+            (None, None) => None,
+        };
+
         Ok(Self {
             ctx,
             addr: listener
                 .local_addr()
                 .expect("Failed to get address from listener"),
             tls_config,
+            auth_token,
+            auth_basic,
         })
     }
 }
@@ -979,19 +1045,31 @@ impl<H: RoapiContext> RunnableServer for RoapiFlightSqlServer<H> {
     }
 
     async fn run(&self) -> Result<(), Whatever> {
-        let router = tonic_server_builder(self.ctx.clone(), &self.tls_config)?;
-        let listener = whatever!(
-            TcpListener::bind(self.addr).await,
-            "Failed to bind address for FlightSQL server",
-        );
+        let svc = FlightServiceServer::new(RoapiFlightSqlService::new(
+            self.ctx.clone(),
+            self.auth_token.clone(),
+            self.auth_basic.clone(),
+        ));
 
-        let incoming = tonic::transport::server::TcpIncoming::from_listener(listener, true, None)
-            .expect("Failed to create incoming TCP Router");
+        let mut builder = tonic::transport::Server::builder();
+        if let Some(cfg) = &self.tls_config {
+            builder = whatever!(
+                builder.tls_config(cfg.clone()),
+                "Failed to build TLS config"
+            );
+        }
+
+        let layer = tower::ServiceBuilder::new()
+            .layer(tower_http::trace::TraceLayer::new_for_grpc())
+            .into_inner();
+
+        let router = builder.layer(layer).add_service(svc);
+
+        router
+            .serve(self.addr)
+            .await
+            .expect("Failed to bind address for FlightSQL server");
 
-        whatever!(
-            router.serve_with_incoming(incoming).await,
-            "Failed to start FlightSQL server on."
-        );
         Ok(())
     }
 }
